/*
 * Sentry Video System
 *   is video server with QoS control from clients' requests.
 *
 *      original by akira-y@is.aist-nara.ac.jp, 1994,1995
 *      modified by k-chinen@is.aist-nara.ac.jp, 1996
 *      modified by satosi-f@is.aist-nara.ac.jp, 1997,2000
 *  
 *  $Id: $
 */

/*
 * sentry.h -- sentry video system general header
 */ 

#ifndef _SENTRY_H_
#define _SENTRY_H_

#include <sys/time.h>

extern int debug;
extern int errno;


/*
 * address, port
 */
#define SENTRY_MULTICAST_ADDR	"224.0.0.1"
#define SENTRY_CTRL_PORT	7001
#define SENTRY_DATA_PORT	7000

/*
 * sentry current version
 */
#define SENTRY_VERSION		3

/*
 * sentry request type
 */
#define I_AM_ALIVE		(0x00)
#define OPEN_UDP_UNICAST	(0xf0)
#define OPEN_UDP_MULTICAST	(0xf1)
#define OPEN_TCP		(0xf2)
#define CLOSE			(0xff)

#define QOS_REQUEST_SIZE	(0x01)
#define QOS_REQUEST_ZOOM	(0x02)
#define QOS_REQUEST_COLOR	(0x04)
#define QOS_REQUEST_FORMAT	(0x08)
#define QOS_REQUEST_RATE	(0x10)

/*
 * video format
 */
#define ENCODE_NONE		(0)
#define ENCODE_JPEG		(1)

/*
 * normal vision ratio (4:3)
 */
#define SIZE_640x480		(1)	/* VGA size */
#define SIZE_320x240		(2)	/* quarter VGA size */
#define SIZE_212x160		(3)	/* quarter CIF size */
#define SIZE_160x120		(4)	
/*
 * wide vision ratio (16:9)
 */
#define SIZE_640x360		(241)
#define SIZE_320x180		(242)
#define SIZE_212x120		(243)
#define SIZE_160x90		(244)

/*
 * color 
 */
#define FULL_COLOR		(1)
#define MONOCHROME		(2)


/*
 * packet header 
 */
typedef struct {
  u_char hd_ver;	/* sentry version */
  u_char type;		/* request type */
  short port;		/* data port# */

  int size;		/* frame size */
  int zoom;		/* zoom ratio */
  int color;		/* colot type */
  int format;		/* video format */
  int rate;		/* frame rate */
} sentry_request_hdr;

/*
 * sentry image header
 */
typedef struct {
#if 0
  struct rtp_hdr;
#endif
  u_char hd_ver;	/* sentry version */
  u_char dummy[3];	/* padding */
  int    offset;	/* image offset */
  union {
    long ts[2];
    struct timeval timestamp; 
  } _hd_ts;             /* time stamp */
#define hd_ts           _hd_ts.ts 
#define hd_tstamp       _hd_ts.timestamp
  int format;		/* video format */
  int size;		/* image size (xsize * ysize * color_space) */
  int width;		/* image xsize */
  int height;		/* image ysize */
  int color;		/* color depth */
} sentry_image_header;


#define	SENTRY_IMAGE_HEADER_LEN	sizeof(sentry_image_header)
#define SENTRY_MAX_DATALEN      (8192-SENTRY_IMAGE_HEADER_LEN)

#endif /* _SENTRY_H_ */
