/*
 * Sentry Video System
 *   is video server with QoS control from clients' requests.
 *
 *      original by akira-y@is.aist-nara.ac.jp, 1994,1995
 *      modified by k-chinen@is.aist-nara.ac.jp, 1996
 *      modified by satosi-f@is.aist-nara.ac.jp, 1997,2000
 *      modified by soichi-n@is.aist-nara.ac.jp, 2002
 *  
 *  $Id:$
 *
 * NOTE:
 *   Early versions of sentry depend on SGI's library.
 *   To instead of SGI's library, this version use libjpeg, standard
 *   JPEG library.
 */

/*
 * image_converter.c - convert images
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#include <jpeglib.h>

#include "sentry.h"

#ifndef MIN
#define MIN(x, y)        ((x) < (y)  ?  (x)  :   (y))
#endif

#define PASS	fprintf(stderr,"PASS %s:%d\n", __FILE__, __LINE__)

#ifndef OUTBUF_SIZE
#define OUTBUF_SIZE      (1024 * 32)
#endif

char *outbuf=NULL;
struct jpeg_compress_struct cinfo;

extern int debug;
extern int verb;

static int jpeg_system_initted = 0;




int
convert_none(char *cur, int size, int xsize, int ysize, int *nxsize, int *nysize, char **new)
{
  *new = cur;
  *nxsize = xsize;
  *nysize = ysize;
  return size;
}

void x_dst_init(j_compress_ptr cinfo)
{
  /*
  PASS;
  */
}

boolean x_dst_isempty(j_compress_ptr cinfo)
{
  /*
  PASS;
  */
  return FALSE;
}

void x_dst_term(j_compress_ptr cinfo)
{
  /*
  PASS;
  */
}

int
init_jpeg_system(int x, int y)
{
  PASS;

  outbuf = (char *) malloc(OUTBUF_SIZE);
  if(!outbuf) {
    fprintf(stderr, "not enough memory\n");
  }
  jpeg_system_initted = 1;

  return 0;
}

int
convert_jpeg(char *cur, int size, int xsize, int ysize,
		int *nxsize, int *nysize, char **new)
{
  struct jpeg_error_mgr jerr;
  int file_index;
  struct jpeg_source_mgr       src_mgr;
  struct jpeg_destination_mgr  dst_mgr;
  JDIMENSION num_scanlines;
  int raw_pos;
  int len;
  JSAMPROW row_pointer[1];

  PASS;
  
  if (jpeg_system_initted == 0) {
    if (init_jpeg_system(xsize, ysize)) {
      return -1;
    }
  }

  dst_mgr.next_output_byte 	= outbuf;
  dst_mgr.free_in_buffer 	= OUTBUF_SIZE;
  dst_mgr.init_destination   	= x_dst_init;
  dst_mgr.empty_output_buffer	= x_dst_isempty;
  dst_mgr.term_destination 	= x_dst_term;
  
  cinfo.err = jpeg_std_error(&jerr);
  
  jpeg_create_compress(&cinfo);
  
  cinfo.dest = &dst_mgr;
  
  fprintf(stderr, "before pos %p, rest %d\n",
	  cinfo.dest->next_output_byte, cinfo.dest->free_in_buffer);
  
  cinfo.image_width = xsize;
  cinfo.image_height = ysize;
  cinfo.input_components = 3;
  cinfo.in_color_space = JCS_RGB;
  jpeg_set_defaults(&cinfo);
  

  fprintf(stderr, "raw RGB pos %p, len %d, size %dx%d\n",
  	cur, size, xsize, ysize);
		    
  jpeg_start_compress(&cinfo, TRUE);
  raw_pos = cinfo.image_width*3;
  while (cinfo.next_scanline < cinfo.image_height) {
    row_pointer[0] = & cur[cinfo.next_scanline * raw_pos];
    jpeg_write_scanlines(&cinfo, row_pointer, 1);
  }
  
  jpeg_finish_compress(&cinfo);
  
  fprintf(stderr, "after  pos %p, rest %d\n",
	  cinfo.dest->next_output_byte, cinfo.dest->free_in_buffer);
  
  *nxsize = xsize;
  *nysize = ysize;
  *new = outbuf;
  len = OUTBUF_SIZE - cinfo.dest->free_in_buffer;

  jpeg_destroy_compress(&cinfo);

  fprintf(stderr, "out %dx%d, addr %p, len %d\n",
    *nxsize, *nysize, *new, len);

  return len;
}
