/*
 * Sentry Video System
 *   is video server with QoS control from clients' requests.
 *
 *      original by akira-y@is.aist-nara.ac.jp, 1994,1995
 *      modified by k-chinen@is.aist-nara.ac.jp, 1996
 *      modified by satosi-f@is.aist-nara.ac.jp, 1997,2000
 *      modified by atsush-m@is.aist-nara.ac.jp, 2002
 *  
 *  $Id: $
 */

/*
 * send_image.c - video handling procedures.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <signal.h>

#include "sentry.h"
#include "sentryd.h"

#include <errno.h>

#define PASS    fprintf(stderr,"PASS %s:%d\n", __FILE__, __LINE__)


extern  fd_set writemask;
extern  int maxwritefds;

extern 	_req_val req_val;

extern  int multicast_clients;
extern  int unicast_clients;

int     frameSize;
int     xsize, ysize;
int     capture_xsize, capture_ysize;
int     xl_offset=0, xr_offset=0, wide_offset;
char   *dataPtr;
char   *new_dataPtr;

void   show_stat();

void
send_image(int signo)
{
  sentry_image_header *hdr;
  struct timeval tv, waittime;
  int new_xsize, new_ysize;
  int fd;
  int datalen, rest, offset, send_datalen, send_copunt;
  int nwrite;
  char *sendptr;
  int encode_type;

#ifdef USE_IOVEC
  sentry_image_header header;
  struct iovec iov[2];
#else
  char packet[8192];
#endif /* USE_IOVEC */
#ifdef ALIVE
  extern void set_alive_mark();
#endif /* ALIVE */
  extern void set_rate();

  PASS;

  if (unicast_clients <= 0 && multicast_clients <= 0) {
    signal(SIGALRM, send_image);
    return;
  }
  signal(SIGALRM, SIG_IGN);

  dataPtr = (char *)capture_frame();
  gettimeofday(&tv, NULL);
#if 1
  fprintf(stderr, "dataPtr %p, len %d\n", dataPtr, frameSize);
#endif

#ifdef ALIVE
  (void)set_alive_mark();
#endif 

  /*
   * clip image for encode from captured image
   */
  if (xl_offset > 0 || xr_offset > 0 || wide_offset > 0) {
    if (clip_image(&dataPtr, &frameSize) < 0) {
      fprintf(stderr, "send_image: cannot clip image.\n");
    }
  }

  /*
   * convert video format
   */
#if 1
  if (debug)
    fprintf(stderr, "send_image: req_val.attr.val[QOS_FORMAT=%d] = %d\n",
	    QOS_FORMAT, req_val.attr.val[QOS_FORMAT]);
#endif

  switch(req_val.attr.val[QOS_FORMAT]) {
  case ENCODE_JPEG:
    datalen = convert_jpeg(dataPtr, frameSize, xsize, ysize, &new_xsize, &new_ysize, &new_dataPtr);
    encode_type = ENCODE_JPEG;
    break;
    
  case ENCODE_NONE:
  default:
    datalen = convert_none(dataPtr, frameSize, xsize, ysize, &new_xsize, &new_ysize, &new_dataPtr);
    encode_type = ENCODE_NONE;
    break;
  }
  
  /* send image */
#ifdef USE_IOVEC
  hdr = &header;
  
  hdr->hd_ver = SENTRY_VERSION;
  hdr->hd_ts[0] = htonl(tv.tv_sec);
  hdr->hd_ts[1] = htonl(tv.tv_usec);
  
  hdr->format = htonl(encode_type);
  hdr->size = htonl(datalen);
  hdr->width = htonl(new_xsize);
  hdr->height = htonl(new_ysize);
  hdr->color = htonl(req_val.attr.val[QOS_COLOR]);
  
  /*
   ** iov
   **  #0  header
   **  #1  body
   */
  iov[0].iov_base = (char *)hdr;
  iov[0].iov_len = sizeof(header);
  
  /* wait a tick. I don't know why :-p */
#ifdef USE_WAITTIME
  waittime.tv_sec = 0;
  waittime.tv_usec = 30000;   /* 30 ms */
#endif
  
  /* 
   * send packets with unicast
   *  (writemask check)
   */
  for(fd=0; fd<=maxwritefds; fd++) {
    if (!FD_ISSET(fd, &writemask)) continue;
#ifdef SUPPORT_MULTIRATE
    /* not implemented yet */
#endif
    
    offset = 0;
    rest = datalen;
    
    sendptr = new_dataPtr;
    /* send_count = 0; */
    
#if 0
    if (debug)
      fprintf(stderr, "send_image: sentptr# %x\n", sendptr);
#endif
    
    while (rest > 0) {
      hdr->offset = htonl(offset);
      
      iov[1].iov_base = sendptr;
      if (rest >= SENTRY_MAX_DATALEN)
	iov[1].iov_len = SENTRY_MAX_DATALEN;
      else 
	iov[1].iov_len = rest;
      
    try_write2:
      nwrite = writev(fd, iov, 2);
      if (nwrite < 0) {
	perror("send_image: writev");
	
	if (errno==EINTR)
	  goto try_write2;
	clear_client(find_client_by_datafd(fd));
	goto fail2;
      } /* have finished sending packets */
      
      send_datalen = nwrite - sizeof(header);
      sendptr = &sendptr[send_datalen];
      offset += send_datalen;
      rest -= send_datalen;
      
#if 0
      if (debug) {
	if (offset == send_datalen || send_datalen < 8156) 
	  fprintf(stderr, "send_image: ts# %d.%06d, rest# %d, send_datalen# %d, nwrite# %d\n",
		  ntohl(hdr->hd_ts[0]), ntohl(hdr->hd_ts[1]), rest, send_datalen, nwrite);
      }
#endif
    } /* while for write-loop */
    
  fail2:;
  }/* for-loop */
  
  /*
   * send packets with (udp) multicast
   */
  if (multicast_clients > 0) {
    hdr->offset = 0;
    rest = datalen;
    
    sendptr = new_dataPtr;
    /* send_count = 0; */
    
    while (rest > 0) {
      iov[1].iov_base = sendptr;
      if (rest >= SENTRY_MAX_DATALEN)
	iov[1].iov_len = SENTRY_MAX_DATALEN;
      else 
	iov[1].iov_len = rest;
      
    try_write:
      nwrite = writev(fd, iov, 2);
      if (nwrite < 0) {
	fprintf(stderr, "send_image: writev: %s\n", sys_errlist[errno]);

	if (errno==EINTR)
	  goto try_write;

	clear_client(find_client_by_datafd(fd));
	goto fail;
      } /* have finished sending packets */
      
      send_datalen = nwrite - sizeof(header);
      hdr->offset += send_datalen;
      sendptr = &sendptr[send_datalen];
      rest -= send_datalen;
    } /* while for write-loop */
    
  fail:;
  }
  
#else /* NOT IOVEC */
  fprintf(stderr, "Now, only USE_IOVEC version is supported.\n");
  exit (1);
#endif /* USE_IOVEC */

  release_frame();
  
  signal(SIGALRM, send_image);
  /*
   * for statistics information
   */
  show_stat();
}

/*
 * clip image
 */
int 
clip_image(char **data, int *size)
{
  int color_depth, i, n, x_offset;
  char *ptr, tmp;

  PASS;
  
  switch(req_val.attr.val[QOS_COLOR]) {
  case FULL_COLOR:
#ifdef IRIX 
    color_depth = 4;
#else
    color_depth = 3;
#endif
    break;
  case MONOCHROME:
  default:
    color_depth = 1;
    break;
  }

#if 0
  fprintf(stderr, "color_depth: %d\n", color_depth);
#endif

  (*size) = xsize * ysize * color_depth;

  if(wide_offset > 0)
    *data = &(*data)[capture_xsize*wide_offset*color_depth];

#ifndef IRIX
  if(xl_offset > 0 || xr_offset > 0){
    x_offset = xl_offset + xr_offset;
    *data = &(*data)[xl_offset*color_depth];
    for(i=0, ptr=&(*data)[xsize*color_depth]; i<ysize; i++){
      for(n=0; n<xsize*color_depth; n++){
        ptr[0] = ptr[x_offset*color_depth];
	ptr++;
      }
      x_offset = x_offset + xl_offset + xr_offset;
    }
  }
#endif
  return 0;
}

#ifdef ALIVE
/*
 * plot alive mark in top line
 */
void
set_alive_mark(void)
{
  int i, color_depth;
  static int alive_pos = 0;

  switch(req_val.attr.val[QOS_COLOR]) {
  case FULL_COLOR:
#ifdef IRIX
    color_depth = 4;
#else
    color_depth = 3;
#endif
    break;
  case MONOCHROME:
  default:
    color_depth = 1;
    break;
  }

  for(i=0; i<color_depth*2; i++) {
    dataPtr[alive_pos+i] = 0xff;
  }

  if (alive_pos+i-1 < xsize*color_depth)
    alive_pos += i;
  else
    alive_pos = 0;
}
#endif /* ALIVE */

/*
 * show send_image statistics information 
 */
void show_stat() 
{
  static int FrameCount = 0;
  static int TotalCount = -1;
  static struct timeval start_tv = {0, 0};
  struct timeval tv;
  
  if (start_tv.tv_sec == 0) 
    gettimeofday(&start_tv, NULL);
  gettimeofday(&tv, NULL);

  if (++TotalCount >= 1000) {
    if (tv.tv_usec -= start_tv.tv_usec) {
      tv.tv_sec -= 1;
      tv.tv_usec += 1000000;
    }
    tv.tv_sec -= start_tv.tv_sec;

    fprintf(stdout, "Total: frame# %d, uptime: %d.%06d s, rate# %.2f fps\n",
	    FrameCount, tv.tv_sec, tv.tv_usec, 
	    (((double)FrameCount)*1000000 / (tv.tv_sec*1000000+tv.tv_usec)));
    TotalCount = 0;
  }
  FrameCount ++;
}
